/*#######################################################################################
AVR DCF77 Clock 

Copyright (C) 2005 Ulrich Radig

#######################################################################################*/

#include "DCF77.h"
#include "clock.h"

//Die Uhrzeit seht in folgenden Variablen
unsigned char ss	= 0;	//Globale Variable fr die Sekunden
unsigned char mm	= 0;	//Globale Variable fr die Minuten
unsigned char hh	= 0;	//Globale Variable fr die Stunden
unsigned char day	= 0;	//Globale Variable fr den Tag
unsigned char mon	= 0;	//Globale Variable fr den Monat
unsigned int year	= 0;	//Globale Variable fr das Jahr

//Bitzhler fr RX Bit
unsigned char rx_bit_counter = 0;

//64 Bit fr DCF77 bentigt werden 59 Bits
unsigned long long dcf_rx_buffer = 0;



//############################################################################
//Overflow Interrupt wird ausgelst bei 59Sekunde oder fehlenden DCF77 Signal 
SIGNAL (SIG_OVERFLOW1)
//############################################################################
{	

		
	//Zurcksetzen des Timers
	TCNT1 = count_res;
	INT0_CONTROL = INT0_FALLING_EDGE;


	//wurden alle 59 Bits empfangen und sind die Paritys richtig?
	if (rx_bit_counter == 59)  	//Alle 59Bits empfangen stellen der Uhr nach DCF77 Buffer
		{
			struct  DCF77_Bits *rx_buffer;
			rx_buffer = (struct DCF77_Bits *)(unsigned long long)&dcf_rx_buffer;

			if (flags.parity_P1 == rx_buffer->P1 && 
				flags.parity_P2 == rx_buffer->P2 &&
				flags.parity_P3 == rx_buffer->P3)
				flags.dcf_allsync = 1;
			else
				flags.dcf_allsync = 0;

			//Berechnung der Minuten BCD to HEX
			if(flags.parity_P1 == rx_buffer->P1)
				mm = rx_buffer->Min-((rx_buffer->Min/16)*6);
			//Berechnung der Stunden BCD to HEX
			if(flags.parity_P2 == flags.parity_P2)
				hh = rx_buffer->Hour-((rx_buffer->Hour/16)*6);
			//Berechnung des Tages BCD to HEX
	
			if(flags.parity_P3 == rx_buffer->P3)
			{
				day= rx_buffer->Day-((rx_buffer->Day/16)*6); 
				//Berechnung des Monats BCD to HEX
				mon= rx_buffer->Month-((rx_buffer->Month/16)*6);
				//Berechnung des Jahres BCD to HEX
				year= 2000 + rx_buffer->Year-((rx_buffer->Year/16)*6);
		
			}

		flags.dcf_sync = 1;
		}
	else
		//nicht alle 59Bits empfangen bzw kein DCF77 Signal 
		{
		flags.dcf_rx = 0; //toggel NS im Display
		flags.dcf_sync = 0; //DCF nicht syncronisiert
		}
	//zurcksetzen des RX Bit Counters
	rx_bit_counter = 0;		
	//Lschen des Rx Buffers
	dcf_rx_buffer = 0;

};

//############################################################################
//DCF77 Modul empfngt Trger 
SIGNAL (DCF77_INT)
//############################################################################
{
	//Auswertung der Pulseweite 
	if (INT0_CONTROL == INT0_FALLING_EDGE) //wenn Fallende flanke
		{
		flags.dcf_rx ^= 1; //toggel RX im Display
	
		//Zurcksetzen des Timers
		TCNT1 = count_res;

		//Nchster Interrupt wird ausgelst bei abfallender Flanke
		INT0_CONTROL = INT0_RISING_EDGE;


		}
	else
		{


		//Auslesen der Pulsweite von ansteigender Flanke zu abfallender Flanke
		unsigned int pulse_wide = TCNT1;

		//beginn von Bereich P1/P2/P3
		if ((rx_bit_counter ==  21) | (rx_bit_counter ==  29) | (rx_bit_counter ==  36)) 
			{
			flags.parity_err = 0;
			}
		//Speichern von P1
		if (rx_bit_counter ==  28) {flags.parity_P1 = flags.parity_err;}
		//Speichern von P2
		if (rx_bit_counter ==  35) {flags.parity_P2 = flags.parity_err;}
		//Speichern von P3
		if (rx_bit_counter ==  58) {flags.parity_P3 = flags.parity_err;}
		//berprfen ob eine 0 oder eine 1 empfangen wurde
		//0 = 100ms
		//1 = 200ms

		//Abfrage grer als 180ms kleiner als 220ms	
		if ((pulse_wide > hi_min) && (pulse_wide < hi_max) )
			{
			//Schreiben einer 1 im dcf_rx_buffer an der Bitstelle rx_bit_counter
			dcf_rx_buffer = (dcf_rx_buffer) | ((unsigned long long) 1 << rx_bit_counter);
			//Toggel Hilfs Parity
			flags.parity_err ^= 1;
		
			}
		//Nchster Interrupt wird ausgelst bei ansteigender Flanke
		INT0_CONTROL = INT0_FALLING_EDGE;	
		//RX Bit Counter wird um 1 incrementiert
		rx_bit_counter++;
		

		}
};


//############################################################################
//Diese Routine startet und inizialisiert den Timer
void Start_Clock (void)
//############################################################################
{
	//Interrupt DCF77 einschalten auf ansteigende Flanke
	DCF77_INT_ENABLE();
	INT0_CONTROL = INT0_FALLING_EDGE;
		
	//Interrupt Overfolw enable
	TIMSK = (1<<TOIE1);
	//Setzen des Prescaler auf 1024 
	TCCR1B |= (1<<CS10 | 0<<CS11 | 1<<CS12); 
	TCNT1 = count_res;
	return;
}

